@@ -1,7 +1,7 @@
 /**
  * @file PyDictProxyHandler.cc
  * @author Caleb Aikens (caleb@distributive.network) and Philippe Laporte (philippe@distributive.network)
- * @brief Struct for creating JS proxy objects. Used by DictType for object coercion TODO
+ * @brief Struct for creating JS proxy objects for Dicts
  * @date 2023-04-20
  *
  * @copyright Copyright (c) 2023-2024 Distributive Corp.
@@ -12,7 +12,6 @@
 
 #include "include/jsTypeFactory.hh"
 #include "include/pyTypeFactory.hh"
-#include "include/StrType.hh"
 
 #include <jsapi.h>
 #include <jsfriendapi.h>
@@ -69,9 +68,12 @@ bool PyDictProxyHandler::set(JSContext *cx, JS::HandleObject proxy, JS::HandleId
   PyObject *attrName = idToKey(cx, id);
 
   PyObject *self = JS::GetMaybePtrFromReservedSlot<PyObject>(proxy, PyObjectSlot);
-  if (PyDict_SetItem(self, attrName, pyTypeFactory(cx, rootedV)->getPyObject())) {
+  PyObject *value = pyTypeFactory(cx, rootedV);
+  if (PyDict_SetItem(self, attrName, value)) {
+    Py_DECREF(value);
     return result.failCantSetInterposed(); // raises JS exception
   }
+  Py_DECREF(value);
   return result.succeed();
 }
 
