@@ -1,7 +1,7 @@
 /**
  * @file PyListProxyHandler.cc
  * @author Philippe Laporte (philippe@distributive.network)
- * @brief Struct for creating JS proxy objects. Used by ListType for List coercion
+ * @brief Struct for creating JS proxy objects for Lists
  * @date 2023-12-01
  *
  * @copyright Copyright (c) 2023-2024 Distributive Corp.
@@ -15,7 +15,6 @@
 #include "include/JSArrayProxy.hh"
 #include "include/JSFunctionProxy.hh"
 #include "include/pyTypeFactory.hh"
-#include "include/StrType.hh"
 
 #include <jsapi.h>
 #include <jsfriendapi.h>
@@ -46,8 +45,9 @@ static bool makeNewPyMethod(JSContext *cx, JS::MutableHandleValue function, JS::
   PyObject *func = PyMethod_Function(method);
   JS::RootedValue thisValue(cx);
   thisValue.setObject(*thisObject);
-  PyObject *newSelf = pyTypeFactory(cx, thisValue)->getPyObject();
+  PyObject *newSelf = pyTypeFactory(cx, thisValue);
   function.set(jsTypeFactory(cx, PyMethod_New(func, newSelf)));
+  Py_DECREF(newSelf);
 
   return true;
 }
@@ -112,9 +112,12 @@ static bool array_push(JSContext *cx, unsigned argc, JS::Value *vp) { // surely
   JS::RootedValue elementVal(cx);
   for (unsigned index = 0; index < numArgs; index++) {
     elementVal.set(args[index].get());
-    if (PyList_Append(self, pyTypeFactory(cx, elementVal)->getPyObject()) < 0) {
+    PyObject *value = pyTypeFactory(cx, elementVal);
+    if (PyList_Append(self, value) < 0) {
+      Py_DECREF(value);
       return false;
     }
+    Py_DECREF(value);
   }
 
   args.rval().setInt32(PyList_GET_SIZE(self));
@@ -161,9 +164,12 @@ static bool array_unshift(JSContext *cx, unsigned argc, JS::Value *vp) { // sure
   JS::RootedValue elementVal(cx);
   for (int index = args.length() - 1; index >= 0; index--) {
     elementVal.set(args[index].get());
-    if (PyList_Insert(self, 0, pyTypeFactory(cx, elementVal)->getPyObject()) < 0) {
+    PyObject *value = pyTypeFactory(cx, elementVal);
+    if (PyList_Insert(self, 0, value) < 0) {
+      Py_DECREF(value);
       return false;
     }
+    Py_DECREF(value);
   }
 
   args.rval().setInt32(PyList_GET_SIZE(self));
@@ -273,7 +279,9 @@ static bool array_indexOf(JSContext *cx, unsigned argc, JS::Value *vp) {
   }
 
   JS::RootedValue elementVal(cx, args[0].get());
-  PyObject *result = PyObject_CallMethod(self, "index", "Oi", pyTypeFactory(cx, elementVal)->getPyObject(), start);
+  PyObject *value = pyTypeFactory(cx, elementVal);
+  PyObject *result = PyObject_CallMethod(self, "index", "Oi", value, start);
+  Py_DECREF(value);
 
   if (!result) {
     PyErr_Clear();
@@ -354,7 +362,8 @@ static bool array_splice(JSContext *cx, unsigned argc, JS::Value *vp) {
   JS::RootedValue elementVal(cx);
   for (int index = 0; index < insertCount; index++) {
     elementVal.set(args[index + 2].get());
-    if (PyList_SetItem(inserted, index, pyTypeFactory(cx, elementVal)->getPyObject()) < 0) {
+    PyObject *value = pyTypeFactory(cx, elementVal);
+    if (PyList_SetItem(inserted, index, value) < 0) {
       return false;
     }
   }
@@ -418,13 +427,19 @@ static bool array_fill(JSContext *cx, unsigned argc, JS::Value *vp) {
   }
 
   JS::RootedValue fillValue(cx, args[0].get());
-  PyObject *fillValueItem = pyTypeFactory(cx, fillValue)->getPyObject();
+  PyObject *fillValueItem = pyTypeFactory(cx, fillValue);
+  bool setItemCalled = false;
   for (int index = actualStart; index < actualEnd; index++) {
+    setItemCalled = true;
     if (PyList_SetItem(self, index, fillValueItem) < 0) {
       return false;
     }
   }
 
+  if (!setItemCalled) {
+    Py_DECREF(fillValueItem);
+  }
+
   // return ref to self
   args.rval().set(jsTypeFactory(cx, self));
   return true;
@@ -547,17 +562,22 @@ static bool array_concat(JSContext *cx, unsigned argc, JS::Value *vp) {
   for (unsigned index = 0; index < numArgs; index++) {
     elementVal.set(args[index].get());
 
-    PyObject *item = pyTypeFactory(cx, elementVal)->getPyObject();
+    PyObject *item = pyTypeFactory(cx, elementVal);
     if (PyObject_TypeCheck(item, &JSArrayProxyType)) {
       // flatten the array only a depth 1
       Py_ssize_t itemLength = JSArrayProxyMethodDefinitions::JSArrayProxy_length((JSArrayProxy *)item);
       for (Py_ssize_t flatIndex = 0; flatIndex < itemLength; flatIndex++) {
         if (!JS_GetElement(cx, *(((JSArrayProxy *)item)->jsArray), flatIndex, &elementVal)) {
+          Py_DECREF(item);
           return false;
         }
-        if (PyList_Append(result, pyTypeFactory(cx, elementVal)->getPyObject()) < 0) {
+        PyObject *value = pyTypeFactory(cx, elementVal);
+        if (PyList_Append(result, value) < 0) {
+          Py_DECREF(item);
+          Py_DECREF(value);
           return false;
         }
+        Py_DECREF(value);
       }
     }
     else if (PyObject_TypeCheck(item, &PyList_Type)) {
@@ -565,15 +585,22 @@ static bool array_concat(JSContext *cx, unsigned argc, JS::Value *vp) {
       Py_ssize_t itemLength = PyList_GET_SIZE(item);
       for (Py_ssize_t flatIndex = 0; flatIndex < itemLength; flatIndex++) {
         if (PyList_Append(result, PyList_GetItem(item, flatIndex)) < 0) {
+          Py_DECREF(item);
           return false;
         }
       }
     }
     else {
-      if (PyList_Append(result, pyTypeFactory(cx, elementVal)->getPyObject()) < 0) {
+      PyObject *value = pyTypeFactory(cx, elementVal);
+      if (PyList_Append(result, value) < 0) {
+        Py_DECREF(item);
+        Py_DECREF(value);
         return false;
       }
+      Py_DECREF(value);
     }
+
+    Py_DECREF(item);
   }
 
   args.rval().set(jsTypeFactory(cx, result));
@@ -621,20 +648,23 @@ static bool array_lastIndexOf(JSContext *cx, unsigned argc, JS::Value *vp) {
   }
 
   JS::RootedValue elementVal(cx, args[0].get());
-  PyObject *element = pyTypeFactory(cx, elementVal)->getPyObject();
+  PyObject *element = pyTypeFactory(cx, elementVal);
   for (int64_t index = start; index >= 0; index--) {
     PyObject *item = PyList_GetItem(self, index);
     Py_INCREF(item);
     int cmp = PyObject_RichCompareBool(item, element, Py_EQ);
     Py_DECREF(item);
     if (cmp < 0) {
+      Py_XDECREF(element);
       return false;
     }
     else if (cmp == 1) {
+      Py_XDECREF(element);
       args.rval().setInt32(index);
       return true;
     }
   }
+  Py_XDECREF(element);
 
   args.rval().setInt32(-1);
   return true;
@@ -1236,7 +1266,6 @@ static uint32_t FlattenIntoArray(JSContext *cx,
   JS::RootedValue elementVal(cx);
 
   for (uint32_t sourceIndex = 0; sourceIndex < sourceLen; sourceIndex++) {
-
     if (PyObject_TypeCheck(source, &JSArrayProxyType)) {
       JS_GetElement(cx, *(((JSArrayProxy *)source)->jsArray), sourceIndex, &elementVal);
     }
@@ -1244,7 +1273,7 @@ static uint32_t FlattenIntoArray(JSContext *cx,
       elementVal.set(jsTypeFactory(cx, PyList_GetItem(source, sourceIndex)));
     }
 
-    PyObject *element = pyTypeFactory(cx, elementVal)->getPyObject();
+    PyObject *element = pyTypeFactory(cx, elementVal);
 
     bool shouldFlatten;
     if (depth > 0) {
@@ -1283,6 +1312,8 @@ static uint32_t FlattenIntoArray(JSContext *cx,
 
       targetIndex++;
     }
+
+    Py_DECREF(element);
   }
 
   return targetIndex;
@@ -1316,7 +1347,7 @@ static uint32_t FlattenIntoArrayWithCallBack(JSContext *cx,
       return false;
     }
 
-    PyObject *element = pyTypeFactory(cx, retVal)->getPyObject();
+    PyObject *element = pyTypeFactory(cx, retVal);
 
     bool shouldFlatten;
     if (depth > 0) {
@@ -1380,6 +1411,8 @@ static uint32_t FlattenIntoArrayWithCallBack(JSContext *cx,
         targetIndex++;
       }
     }
+
+    Py_DECREF(element);
   }
 
   return targetIndex;
@@ -2091,7 +2124,7 @@ bool PyListProxyHandler::defineProperty(
   }
 
   JS::RootedValue itemV(cx, desc.value());
-  PyObject *item = pyTypeFactory(cx, itemV)->getPyObject();
+  PyObject *item = pyTypeFactory(cx, itemV);
   PyObject *self = JS::GetMaybePtrFromReservedSlot<PyObject>(proxy, PyObjectSlot);
   if (PyList_SetItem(self, index, item) < 0) {
     // we are out-of-bounds and need to expand
